import { Address } from '../../crypto';
import { Transaction } from '../../transaction/transaction';
import { Transfer } from '../../transaction/transfer';
import { State } from './token';
export declare const GAS_CONTRACT = "0000000000000000000000000000000000000002";
/**
 * Get the address of native asset contract
 * @param tokenType Token type. Can only be DNA or ONG
 */
export declare function getTokenContract(tokenType: string): Address;
/**
 * Verify amount
 * @param amount Amount
 */
export declare function verifyAmount(amount: number | string): void;
/**
 * Creates transaction to transfer native assets.
 * @param tokenType gas
 * @param from sender's address
 * @param to receiver's address
 * @param amount Amount of amount to transfer
 * @param gasPrice Gas price
 * @param gasLimit Gas limit
 * @param payer Address to pay for transaction's gas.
 */
export declare function makeTransferTx(tokenType: string, from: Address, to: Address, amount: number | string, gasPrice: string, gasLimit: string, payer?: Address): Transfer;
/**
 * Create transaction for asset transfer approve
 * @param tokenType gas
 * @param from Sender's address
 * @param to receiver's address
 * @param amount Amount to approve
 * @param gasPrice Gas price
 * @param gasLimit Gas limit
 * @param payer Address of fee payer
 */
export declare function makeApproveTx(tokenType: string, from: Address, to: Address, amount: number | string, gasPrice: string, gasLimit: string, payer?: Address): Transaction;
/**
 * Create transaction for asset transfer from
 * @param tokenType gas
 * @param from Sender's address
 * @param to receiver's address
 * @param amount Amount to approve
 * @param gasPrice Gas price
 * @param gasLimit Gas limit
 * @param payer Address of fee payer
 */
export declare function makeTransferFromTx(tokenType: string, sender: Address, from: Address, to: Address, amount: number | string, gasPrice: string, gasLimit: string, payer?: Address): Transaction;
/**
 * transfer from multiple senders to one receiver
 * this tx needs multiple senders' signature.
 * @param tokenType
 * @param from array of senders' address
 * @param to receiver's address
 * @param amounts
 */
export declare function makeTransferStateTx(tokenType: string, states: State[], gasPrice: string, gasLimit: string, payer?: Address): Transaction;
/**
 * transfer from one sender to multiple receivers
 * @param tokenType
 * @param from
 * @param to
 * @param amounts
 */
export declare function makeTransferToMany(tokenType: string, from: Address, to: Address[], amounts: string | number[], gasPrice: string, gasLimit: string): Transaction;
/**
 * Creates transaction to query allowance that can be sent from sender to receiver
 * @param asset Asset type. Only DNA or ONg.
 * @param from Sender's address
 * @param to Receiver's address
 */
export declare function makeQueryAllowanceTx(from: Address, to: Address): Transaction;
/**
 * Creates transaction to query balance.
 * @param address Address to query balance
 */
export declare function makeQueryBalanceTx(address: Address): Transaction;
export declare function deserializeTransferTx(str: string): Transfer;
